/*
 * (c) Collin R. Mulliner <collin(AT)betaversion.net> 
 * http://www.mulliner.org/collin/
 *
 *
 * This is heavily based on gkfreq by Brad Davis!
 * 
 */

#include <gkrellm2/gkrellm.h>
#include "data.h"

#ifdef FAN
#define	CONFIG_NAME	"gkibm_fan"
#define	STYLE_NAME	"gkibm_fan"
#endif

#ifdef THERMAL_CPU 
#define	CONFIG_NAME	"gkibm_cpu"
#define	STYLE_NAME	"gkibm_cpu"
#endif

#ifdef THERMAL_HDD
#define	CONFIG_NAME	"gkibm_hdd"
#define	STYLE_NAME	"gkibm_hdd"
#endif

#ifdef THERMAL_PCI
#define	CONFIG_NAME	"gkibm_pci"
#define	STYLE_NAME	"gkibm_pci"
#endif

#ifdef THERMAL_GPU
#define	CONFIG_NAME	"gkibm_gpu"
#define	STYLE_NAME	"gkibm_gpu"
#endif

#ifdef THERMAL_BAT
#define	CONFIG_NAME	"gkibm_bat"
#define	STYLE_NAME	"gkibm_bat"
#endif

static GkrellmMonitor	*monitor;
static GkrellmPanel	*panel;
static GkrellmDecal	*decal_text1;
static gchar	*info; 
static gint	style_id;


static gint panel_expose_event(GtkWidget *widget, GdkEventExpose *ev)
{
	gdk_draw_pixmap(widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		panel->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
		ev->area.width, ev->area.height);

	return(FALSE);
}


static void update_plugin()
{
	static gint	x_scroll, w;

	if (w == 0) w = gkrellm_chart_width();
	x_scroll = (x_scroll + 1) % (2 * w);
	
	// dont do it too much...
	if ((GK.timer_ticks % 20) != 0) return;

	#ifdef FAN
	info = read_fan();
	#endif

	#ifdef THERMAL_CPU 
	info = read_data(0);
	#endif

	#ifdef THERMAL_PCI 
	info = read_data(1);
	#endif

	#ifdef THERMAL_HDD 
	info = read_data(2);
	#endif

	#ifdef THERMAL_GPU 
	info = read_data(3);
	#endif

	#ifdef THERMAL_BAT
	info = read_data(4);
	#endif

	decal_text1->x_off = (w - gdk_string_width(decal_text1->text_style.font, info)) / 2;
	if (decal_text1->x_off < 0) decal_text1->x_off = 0;

	gkrellm_draw_decal_text(panel, decal_text1, info, w - x_scroll);
	gkrellm_draw_panel_layers(panel);

	// malloc'd it from read_data/fan
	free(info);
}


static void create_plugin(GtkWidget *vbox, gint first_create) 
{
	GkrellmStyle *style;
	GkrellmTextstyle *ts, *ts_alt;


	if (first_create) panel = gkrellm_panel_new0();

	style = gkrellm_meter_style(style_id);

	ts = gkrellm_meter_textstyle(style_id);
	ts_alt = gkrellm_meter_alt_textstyle(style_id);

	decal_text1 = gkrellm_create_decal_text(panel, "8MHz", ts, style, -1, -1, -1);

	gkrellm_panel_configure(panel, NULL, style);
	gkrellm_panel_create(vbox, monitor, panel);

	if (first_create) {
		g_signal_connect(G_OBJECT (panel->drawing_area), "expose_event", G_CALLBACK (panel_expose_event), NULL);
	}
}


static GkrellmMonitor	plugin_mon = {

	CONFIG_NAME,   /* Name, for config tab. */
	0,             /* Id,  0 if a plugin */
	create_plugin, /* The create function */
	update_plugin, /* The update function */
	NULL,          /* The config tab create function */
	NULL,          /* Apply the config function */

	NULL,         /* Save user config */
	NULL,         /* Load user config */
	NULL,         /* config keyword */

	NULL,         /* Undefined 2 */
	NULL,         /* Undefined 1 */
	NULL,         /* private */

	MON_INSERT_AFTER|MON_CPU, /* Insert plugin before this monitor */

	NULL, /* Handle if a plugin, filled in by GKrellM  */
	NULL  /* path if a plugin, filled in by GKrellM  */
};


GkrellmMonitor* gkrellm_init_plugin()
{
	style_id = gkrellm_add_meter_style(&plugin_mon, STYLE_NAME);
	monitor = &plugin_mon;
	return(&plugin_mon);
}
